@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 2005, 2006, 2009, 2011 by                                     */
+/*  Copyright 2005, 2006, 2009, 2011-2013 by                                */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -21,6 +21,7 @@
 #include FT_CACHE_MANAGER_H
 
 #include FT_GLYPH_H
+#include FT_STROKER_H
 #include FT_BITMAP_H
 
 #include <stdlib.h>
@@ -57,6 +58,13 @@
   /*************************************************************************/
 
 
+  /* default window dimensions */
+#define DIM_X  640
+#define DIM_Y  480
+
+  /* baseline distance between header lines */
+#define HEADER_HEIGHT  12
+
 #include "graph.h"
 #include "grobjs.h"
 #include "grfont.h"
@@ -67,12 +75,15 @@
     grBitmap*   bitmap;
     grColor     fore_color;
     grColor     back_color;
+    grColor     warn_color;
 
   } FTDemo_Display;
 
 
   FTDemo_Display*
-  FTDemo_Display_New( grPixelMode  mode );
+  FTDemo_Display_New( grPixelMode  mode,
+                      int          width,
+                      int          height );
 
 
   void
@@ -178,7 +189,6 @@
     int             hinted;            /* is glyph hinting active?    */
     int             antialias;         /* is anti-aliasing active?    */
     int             use_sbits;         /* do we use embedded bitmaps? */
-    int             low_prec;          /* force low precision         */
     int             autohint;          /* force auto-hinting          */
     int             lcd_mode;
     int             preload;           /* force font file preloading  */
@@ -191,13 +201,14 @@
     int             string_reload;
 
     FT_Encoding     encoding;
+    FT_Stroker      stroker;
     FT_Bitmap       bitmap;            /* used as bitmap conversion buffer */
 
   } FTDemo_Handle;
 
 
   FTDemo_Handle*
-  FTDemo_New( FT_Encoding  encoding );
+  FTDemo_New( void );
 
 
   void
@@ -222,11 +233,6 @@
   FTDemo_Set_Current_Size( FTDemo_Handle*  handle,
                            int             pixel_size );
 
-  void
-  FTDemo_Set_Current_Pointsize( FTDemo_Handle*  handle,
-                                int             point_size,
-                                int             res );
-
   void
   FTDemo_Set_Current_Charsize( FTDemo_Handle*  handle,
                                int             point_size,
